"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.start = void 0;
const net_1 = require("net");
const portUtils_1 = require("../portUtils");
const utils_1 = require("./utils");
const constants_1 = require("../constants");
const spawn_1 = require("../../init/spawn");
const config_1 = require("./config");
async function start() {
    const hostname = constants_1.DEFAULT_HOST;
    let port = constants_1.DEFAULT_PORTS.apphosting;
    while (!(await availablePort(hostname, port))) {
        port += 1;
    }
    serve(port);
    return { hostname, port };
}
exports.start = start;
async function serve(port) {
    const rootDir = process.cwd();
    const packageManager = await (0, utils_1.discoverPackageManager)(rootDir);
    const apphostingLocalConfig = await (0, config_1.getLocalAppHostingConfiguration)(rootDir);
    await (0, spawn_1.wrapSpawn)(packageManager, ["run", "dev"], rootDir, Object.assign(Object.assign({}, apphostingLocalConfig.environmentVariables), { PORT: port }));
}
function availablePort(host, port) {
    return (0, portUtils_1.checkListenable)({
        address: host,
        port,
        family: (0, net_1.isIPv4)(host) ? "IPv4" : "IPv6",
    });
}
